#!/bin/bash

set -e -o pipefail

# General Java options
JAVA_OPTS+=" -showversion"
JAVA_OPTS+=" -XX:+PrintFlagsFinal"
JAVA_OPTS+=" -XX:+PrintCommandLineFlags"
JAVA_OPTS+=" -XX:+PrintStringTableStatistics"
JAVA_OPTS+=" -XX:ErrorFile=/tmp/error_java_identity_%p.log"
JAVA_OPTS+=" -Xlog:gc*:/tmp/migration.txt:time,level,tags:filesize=50M,filecount=1"
JAVA_OPTS+=" -XX:-OmitStackTraceInFastThrow"

# memory options
# use parallel GC if no GC is added in JAVA_OPTS - the default G1 uses too much off-heap memory
if ! grep -E 'UseG1GC|UseConcMarkSweepGC|UseEpsilonGC|UseParallelGC|UseSerialGC|UseZGC' <<< "$JAVA_OPTS" > /dev/null; then
  echo "Setting GC to ParallelGC"
  JAVA_OPTS+=" -XX:+UseParallelGC"
fi

JAVA_OPTS+=" -Xms${JVM_XMS:-768m} -Xmx${JVM_XMX:-1024m}"

echo "Setting metaspace to ${JVM_METASPACE:=256m}"
JAVA_OPTS+=" -XX:MaxMetaspaceSize=$JVM_METASPACE"

JAVA_OPTS+=" -Xss384k"
JAVA_OPTS+=" -XX:MaxJavaStackTraceDepth=0"
JAVA_OPTS+=" -XX:SurvivorRatio=8"
JAVA_OPTS+=" -XX:TargetSurvivorRatio=90"
JAVA_OPTS+=" -XX:MaxTenuringThreshold=15"
JAVA_OPTS+=" -XX:ParallelGCThreads=4"
JAVA_OPTS+=" -XX:+OptimizeStringConcat"

HEAP_DUMP_PATH="/heapDump/$( date '+%Y-%m-%d_%H-%M-%S' )_heap_dump.hprof"
JAVA_OPTS+=" -XX:+HeapDumpOnOutOfMemoryError -XX:HeapDumpPath=$HEAP_DUMP_PATH"
JAVA_OPTS+=" -XX:+ExitOnOutOfMemoryError"

if [ -n "$JMX_PORT" ]; then
  echo "Enabling JMX connection on port: $JMX_PORT"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.port=$JMX_PORT"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.rmi.port=$JMX_PORT"
  JAVA_OPTS+=" -Djava.rmi.server.hostname=${JMX_HOST:-127.0.0.1}"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.ssl=false"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.local.only=false"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.authenticate=false"
fi

# General service variables
if [ -n "$AUDIT_LOG_MODE" ]; then
    JAVA_OPTS+=" -DauditLog.mode=$AUDIT_LOG_MODE"
fi

if [ -n "$ENV_NAMES" ]; then
    JAVA_OPTS+=" -DauditLog.env.names=$ENV_NAMES"
fi

# Opensearch variables
if [ -n "$OPENSEARCH_PROTOCOL" ]; then
    JAVA_OPTS+=" -DauditLog.opensearch.protocol=$OPENSEARCH_PROTOCOL"
fi

if [ -n "$OPENSEARCH_HOST" ]; then
    JAVA_OPTS+=" -DauditLog.opensearch.host=$OPENSEARCH_HOST"
fi

if [ -n "$OPENSEARCH_PORT" ]; then
    JAVA_OPTS+=" -DauditLog.opensearch.port=$OPENSEARCH_PORT"
fi

if [ -n "$OPENSEARCH_USER" ]; then
    JAVA_OPTS+=" -DauditLog.opensearch.user=$OPENSEARCH_USER"
fi

if [ -n "$OPENSEARCH_PASS" ]; then
    JAVA_OPTS+=" -DauditLog.opensearch.pass=$OPENSEARCH_PASS"
fi

if [ -n "$OPENSEARCH_TIMEOUT" ]; then
    JAVA_OPTS+=" -DauditLog.opensearch.timeoutMillis=$OPENSEARCH_TIMEOUT"
fi

# Connector and external services variables
if [ -n "$CONNECTOR_ID" ]; then
    JAVA_OPTS+=" -DauditLog.web.server.connectorId=$CONNECTOR_ID"
fi

if [ -n "$CONNECTOR_KEY" ]; then
    JAVA_OPTS+=" -DauditLog.web.server.connectorKey=$CONNECTOR_KEY"
fi

if [ -n "$MULTI_ENV_CONNECTOR_MAPPING" ]; then
    JAVA_OPTS+=" -DauditLog.web.server.multiEnvConnectorMapping=$(echo -n $MULTI_ENV_CONNECTOR_MAPPING | tr -d ' ')"
fi

if [ -n "$MULTI_ENV_HOST_MAPPING" ]; then
    JAVA_OPTS+=" -DauditLog.web.server.multiEnvHostMapping=$(echo -n $MULTI_ENV_HOST_MAPPING | tr -d ' ')"
fi

if [ -n "$INTERNAL_GATEWAY_HOSTNAME" ]; then
    JAVA_OPTS+=" -DauditLog.web.client.internalGatewayHostName=$INTERNAL_GATEWAY_HOSTNAME"
fi

if [ -n "$SECOPS_S3_SECRET" ]; then
    echo -n $SECOPS_S3_SECRET | jq -Mr .
    JAVA_OPTS+=" -DauditLog.aws.s3.secops.accessKey=$(echo -n $SECOPS_S3_SECRET | jq -Mr '."access-key"')"
    JAVA_OPTS+=" -DauditLog.aws.s3.secops.secretKey=$(echo -n $SECOPS_S3_SECRET | jq -Mr '."secret-key"')"
    JAVA_OPTS+=" -DauditLog.aws.s3.secops.sseksmKeyId=$(echo -n $SECOPS_S3_SECRET | jq -Mr '."ssekms-key-id"')"
    JAVA_OPTS+=" -DauditLog.aws.s3.secops.acl=$(echo -n $SECOPS_S3_SECRET | jq -Mr '."acl"')"
fi

if [ -n "$SECOPS_S3_REGION" ]; then
    JAVA_OPTS+=" -DauditLog.aws.s3.secops.region=$SECOPS_S3_REGION"
fi

if [ -n "$SECOPS_S3_BUCKET_NAME" ]; then
    JAVA_OPTS+=" -DauditLog.aws.s3.secops.bucketName=$SECOPS_S3_BUCKET_NAME"
fi

if [ -n "$EXT_JAVA_OPTS" ]; then
  echo "External java options: $EXT_JAVA_OPTS"
  JAVA_OPTS+=" $EXT_JAVA_OPTS"
fi

echo "JAVA_OPTS: $JAVA_OPTS"

JAR_NAME="audit-log-server.jar"

echo -e '\n\n==============\n\n\n'

cd "$AUDIT_LOG_SERVER"
java \
    $JAVA_OPTS \
    -jar $JAR_NAME \
    "$@" &
PID=$!

EXITCODE=0
wait "$PID" || EXITCODE=$?
echo "Java process exited with $EXITCODE"
exit "$EXITCODE"